define([
  "dojo/_base/declare",
  "dojo/_base/window",
  "dojo/dom-attr",
  "dojo/dom-class",
  "dojo/on",
  "dijit/_WidgetBase",
  "dijit/_TemplatedMixin",
  "dijit/form/TimeTextBox",
  "dojo/text!./templates/DateTimeTextBox.html",
  "wagby/DateTextBox"
], function(declare, win, domAttr, domClass, on, _WidgetBase, _TemplatedMixin, TimeTextBox, template, DateTextBox) {
  return declare("wagby.DateTimeTextBox", [_WidgetBase, _TemplatedMixin], {
    templateString: template,
    dateFormat: "yyyy-MM-dd",
    timeFormat: "HH:mm:ss",
    dateTimeFormat: "yyyy-MM-dd HH:mm:ss",
    DEFAULT_TIME_VALUE: "00:00:00",
    DEFAULT_TIME_FORMAT: "HH:mm:ss",
    DATETIME_FORMAT_RE: /^\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}$/,
    pickerMin: "T00:00:00",
    pickerMax: "T23:59:59",
    timeIncrement: "T00:15:00",
    timeSelectInputRE: /^HM:(\d+)-(\d+):(\d+)$/,
    enterKeyAction: null,
    submitOnChange: false,
    rangeType: "",

    postCreate: function() {
      this.inherited(arguments);
      this.containerNode.setAttribute("style", "white-space: nowrap;");
      this.initDateTextBox();
      this.initTimeTextBox();
      this.setEnterKeyActionEvent();
      this.setErrorStyle();
    },

    _setIdAttr: function(id) {
      this.containerNode.setAttribute("id", id);
      this.valueNode.setAttribute("id", id + "_value");
    },

    _setNameAttr: function(name) {
      if(this.valueNode){
        this.valueNode.setAttribute("name", name);
      }
    },

    _setValueAttr: function(value) {
      var dateTimeValue;
      if (value instanceof Date) {
        try {
          dateTimeValue = _setDateWithFormat(value, this.dateTimeFormat);
        } catch(e) {
        }
      } else {
        dateTimeValue = value;
      }
      if (!dateTimeValue) {
        return;
      }
      if (dateTimeValue.indexOf(" ") === -1) {
        dateTimeValue = dateTimeValue + " " + this.DEFAULT_TIME_VALUE;
      }
      this.valueNode.value = dateTimeValue;
      this.setValueToWidget(dateTimeValue);
    },

    setValueToWidget: function(dateTimeValue) {
      if (!this.dateTextBox.set || !this.timeTextBox.set) {
        return;
      }
      this.dateTextBox.set("displayedValue", _setDateWithFormat(dateTimeValue, this.dateFormat));
      var dateTimeValues = dateTimeValue.split(" ");
      this.timeTextBox.set("displayedValue", dateTimeValues[1]);
    },

    _getValueAttr: function() {
      if (!this.valueNode.value || this.valueNode.value === "") {
        return "";
      }
      if (this.valueNode.value.indexOf(" ") === -1) {
        this.valueNode.value = this.valueNode.value + " " + this.DEFAULT_TIME_VALUE;
      }
      return this.valueNode.value;
    },

    _setTimeSelectInputAttr: function(value) {
      if (!value) {
        return;
      }
      var props = this.timeSelectInputRE.exec(value);
      if (!props || props.length !== 4) {
        return;
      }
      this.pickerMin = this.getPickerTime(props[1]);
      this.pickerMax = this.getPickerTime(props[2]);
      this.timeIncrement = this.getTimeIncrementValue(props[3]);
    },

    _setDisabledAttr: function(value, priorityChange) {
      this.dateTextBox.set("disabled", value);
      this.timeTextBox.set("disabled", value);
      this.valueNode.disabled = value;
    },

    _setDateTimeValueAttr: function(value) {
      if (!value || !this.DATETIME_FORMAT_RE.test(value)) {
        return;
      }
      this.dateTextBox.set("displayedValue",
                           _setDateWithFormat(value, this.dateFormat));
      var dateTimeValue = value;
      var timeValue = dateTimeValue.substring(dateTimeValue.indexOf(" ") + 1);
      this.timeTextBox.set("displayedValue", timeValue);
    },

    _setReadOnlyAttr: function(value, priorityChange) {
      this.dateTextBox.set("readOnly", value);
      this.timeTextBox.set("readOnly", value);
    },

    getPickerTime: function(hour) {
      if (hour === "24") {
        return "T23:59:59";
      }
      return "T" + this.getPaddingValue(hour) + ":00:00";
    },

    getTimeIncrementValue: function(value) {
      var timeIncrementValue = value;
      if (timeIncrementValue === "0") {
        timeIncrementValue = "1";
      }
      return "T00:" + this.getPaddingValue(timeIncrementValue) + ":00";
    },

    getPaddingValue: function(value) {
      return ("0" + value).substr(-2);
    },

    initDateTextBox: function() {
      var dateValue = _setDateWithFormat(this.value, this.dateFormat);
      this.dateTextBox = new DateTextBox({
        'id': this.id + "_DateTextBox",
        'name': this.id + "_DateTextBox",
        'style': this.style,
        'value': dateValue,
      }).placeAt(this.dateTextBox);
      if (!this.dateFormat || this.dateFormat === "") {
        this.dateFormat = this.dateTextBox.DEFAULT_DATE_FORMAT;
      }
      this.dateTextBox.set("dateFormat", this.dateFormat);
      this.dateTextBox.set("displayedValue", dateValue);
      if (this.params.dayOffset) {
        this.dateTextBox.set("dayOffset", this.params.dayOffset);
      }
      if (this.style) {
        this.dateTextBox.set("style", this.style);
      }
      if (this.rangeType) {
        this.dateTextBox.set("rangeType", this.rangeType);
      }
      this.dateTextBox.set("style", "width: " + this.dateTextBox.getWidthByDateFormat(this.dateFormat));
      var that = this;
      this.dateTextBox.on("change", function(value) {
        if (!value) {
          that.timeTextBox.reset();
          return;
        }
        that.setFocusTimeTextBox();
        that.setConditionDefaultTimeValue();
      });
      on(this.dateTextBox.textbox, "blur", function() {
        that.autoFormatDate();
      });
      this.dateTextBox.on("focus", function() {
        setScrollPosition(this.id);
      });
      this.setCommonProperties(this.dateTextBox);
    },

    setConditionDefaultTimeValue: function() {
      if (this.rangeType === "") {
        return;
      }
      if (this.rangeType === "begin" &&
          this.timeTextBox.get("displayedValue") === "") {
        this.timeTextBox.set("value", "T00:00:00");
      } else if (this.rangeType === "end" &&
                 this.timeTextBox.get("displayedValue") === "") {
        this.timeTextBox.set("value", "T23:59:59");
      }
    },

    setFocusTimeTextBox: function() {
      this.timeTextBox.focus();
    },

    autoFormatDate: function() {
      var inputDateValue = this.dateTextBox.textbox.value;
      if (!inputDateValue || inputDateValue === "" || this.dateTextBox.isValid()) {
        return;
      }
      this.dateTextBox._setValueAttr(inputDateValue);
    },

    initTimeTextBox: function() {
      var dateTimeValue = this._getValueAttr();
      var timeValue = dateTimeValue.substring(dateTimeValue.indexOf(" ") + 1);
      this.timeTextBox = new TimeTextBox({
        'id': this.id + "_TimeTextBox",
        'name': this.id + "_TimeTextBox",
        'style': this.style,
        'value': timeValue,
      }).placeAt(this.timeTextBox);
      if (!this.timeFormat || this.timeFormat === "") {
        this.timeFormat = this.DEFAULT_TIME_FORMAT;
      }
      this.timeTextBox.set("constraints", {
        timePattern: this.timeFormat,
        pickerMin: this.pickerMin,
        pickerMax: this.pickerMax,
        clickableIncrement: this.timeIncrement,
      });
      this.timeTextBox.set("displayedValue", timeValue);
      this.timeTextBox.set("style", "width: " + this.getWidthFromTimeFormat(this.timeFormat));
      var that = this;
      this.timeTextBox.on("change", function() {
        that.complementTime();
      });
      this.timeTextBox.on("focus", function() {
        setScrollPosition(this.id);
      });
      this.setCommonProperties(this.timeTextBox);
      // タブ移動で時刻入力欄を囲む div タグにフォーカスがセットされないよう tabindex 属性を取り除く
      domAttr.remove(this.focusNode, "tabindex");
    },

    complementTime: function() {
      var inputTimeValue = this.timeTextBox.textbox.value;
      var inputTimeLength = inputTimeValue.length;
      if (inputTimeLength === this.timeFormat.length) {
        return;
      }
      var complementTimeValue = convText2Date(_setDateWithFormat(this.dateTextBox.get("value"), this.dateTextBox.DEFAULT_DATE_FORMAT) + " " + inputTimeValue);
      this.timeTextBox.set("_onChangeActive", false);
      this.timeTextBox.set("value", "T" + _setDateWithFormat(complementTimeValue, this.DEFAULT_TIME_FORMAT));
      this.timeTextBox.set("_onChangeActive", true);
      this.timeTextBox.closeDropDown();
      this.setDateTimeValue();
    },

    setCommonProperties: function(widget) {
      var that = this;
      widget.on("change", function() {
        that.setDateTimeValue();
      });
      if (this.params.onKeyDown) {
        widget.on("keydown", function(event) {
          that.params.onKeyDown(event);
        });
      }
      if (this.params.onChange) {
        widget.on("change", function(event) {
          that.params.onChange(event);
        });
      }
      if (this.params.required) {
        widget.set("required", true);
      }
      if (this.params.tabIndex) {
        widget.set("tabIndex", this.params.tabIndex);
      }
    },

    setEnterKeyActionEvent: function() {
      if (!this.enterKeyAction) {
        return;
      }
      var keydownFunc = this.getEnterKeyActionFunc();
      if (!keydownFunc) {
        return;
      }
      this.dateTextBox.on("keydown", keydownFunc);
      this.timeTextBox.on("keydown", keydownFunc);
    },

    getEnterKeyActionFunc: function() {
      var keydownFunc;
      var that = this;
      if (this.enterKeyAction === "changetab") {
        this.dateTextBox.on("keydown", function(event) {
          if (event.keyCode !== 0x0d) return;
          event.preventDefault();
          that.timeTextBox.focus();
        });
        this.timeTextBox.on("keydown", function(event) {
          if (event.keyCode !== 0x0d) return;
          event.preventDefault();
          enterKeyHandler.focusNextElement(that, event);
        });
        return;
      } else if (this.enterKeyAction === "ignore") {
        keydownFunc = function(event) {
          if (event.keyCode !== 0x0d) return;
          event.preventDefault();
        }
      } else if (this.enterKeyAction === "default") {
        keydownFunc = function(event) {
          if (event.keyCode !== 0x0d) return;
          doSubmitForm(event.srcElement.form);
        }
      }
      return keydownFunc;
    },

    setDateTimeValue: function() {
      var oldDateTimeValue = _setDateWithFormat(this.valueNode.value, this.dateTimeFormat);
      var dateValue = this.dateTextBox.get("value");
      if (!dateValue || dateValue === "") {
        this.valueNode.value = "";
        this.timeTextBox.set("value", "");
        this.timeTextBox.set("displayedValue", "");
        this.submitOnChangeFunc(oldDateTimeValue, this.valueNode.value);
        return;
      }
      var timeValue = this.timeTextBox.get("value");
      if (!timeValue || timeValue === "") {
        timeValue = this.DEFAULT_TIME_VALUE;
      }
      this.valueNode.value = _setDateWithFormat(this.dateTextBox.get("value"), this.dateTextBox.DEFAULT_DATE_FORMAT)
        + " " + _setDateWithFormat(timeValue, this.DEFAULT_TIME_FORMAT);
      this.submitOnChangeFunc(oldDateTimeValue, this.valueNode.value);
    },

    submitOnChangeFunc: function(oldValue, newValue) {
      if (!this.submitOnChange || oldValue === newValue) {
        return;
      }
      var dateValue = this.dateTextBox.get("displayedValue");
      var timeValue = this.timeTextBox.get("displayedValue");
      if (!(dateValue === "" && timeValue === "") && timeValue === "") {
        return;
      }
      this.dateTextBox.textbox.form.submit();
    },

    getWidthFromTimeFormat: function(format) {
      var width = format.length;
      var theme = domAttr.get(win.body(), "class");
      if (format.length <= 8 && theme.lastIndexOf("flat_", 0) === 0) {
        width = width + 2;
      } else if (format.length > 8) {
        width = this.dateTextBox.getWidthByTheme(format, width);
      }
      return width + "em";
    },

    reset: function(){
      this.dateTextBox.set("value", "");
      this.timeTextBox.set("value", "");
      this.valueNode.value = "";
      this.inherited(arguments);
    },

    focus: function() {
      var focusWidget = this.dateTextBox.textbox;
      if (this.dateTextBox.get("value") && this.dateTextBox.get("value") !== "" &&
          (!this.timeTextBox.get("value") || this.timeTextBox.get("value") === "")) {
        focusWidget = this.timeTextBox.textbox;
      }
      focusWidget.focus();
    },

    validate: function(isFocused){
      return this.disabled || this.isValid(isFocused);
    },

    isValid: function(isFocused){
      if (!this.required) {
        return true;
      }
      return this.dateTextBox.get("value") && this.dateTextBox.get("value") !== "" &&
        this.timeTextBox.get("value") && this.timeTextBox.get("value") !== "";
    },

    setErrorStyle: function() {
      if (!this.inputError || this.inputError === "false") {
        return;
      }
      domClass.add(this.dateTextBox.domNode, "errorfield");
      domClass.add(this.timeTextBox.domNode, "errorfield");
    },

  });
});
